import Prism from "prismjs";
import "prismjs/plugins/custom-class/prism-custom-class";
Prism.plugins.customClass.map({ number: "prism-number", tag: "prism-tag" });

// codeInput
// by WebCoder49
// Based on a CSS-Tricks Post
// Needs Prism.js

var codeInput = {
  update: function(text, code_input) {
    code_input.setAttribute("value", text);
    let result_element = code_input.getElementsByClassName("code-input_highlighting-content")[0];
    // Handle final newlines (see article)
    if(text[text.length-1] == "\n") {
      text += " ";
    }
    // Update code
    result_element.innerHTML = text.replace(new RegExp("&", "g"), "&amp;").replace(new RegExp("<", "g"), "&lt;"); /* Global RegExp */
    // Syntax Highlight
    Prism.highlightElement(result_element);
  },

  sync_scroll: function(element, code_input) {
    /* Scroll result to scroll coords of event - sync with textarea */
    let result_element = code_input.getElementsByClassName("code-input_highlighting")[0];
    // Get and set x and y
    result_element.scrollTop = element.scrollTop;
    result_element.scrollLeft = element.scrollLeft;
  },

  check_tab: function(element, event) {
    let code = element.value;
    if(event.key == "Tab") {
      /* Tab key pressed */
      event.preventDefault(); // stop normal
      let before_tab = code.slice(0, element.selectionStart); // text before tab
      let after_tab = code.slice(element.selectionEnd, element.value.length); // text after tab
      let cursor_pos = element.selectionEnd + 1; // where cursor moves after tab - moving forward by 1 char to after tab
      element.value = before_tab + "\t" + after_tab; // add tab char
      // move cursor
      element.selectionStart = cursor_pos;
      element.selectionEnd = cursor_pos;
    }
  }
}
window.codeInput = codeInput

class CodeInput extends HTMLElement { // Create code input element
  constructor() {
    super(); // Element
  }
  connectedCallback() {
    // Added to document
    
    /* Defaults */
    let lang = this.getAttribute("lang") || "HTML";
    let placeholder = this.getAttribute("placeholder") || "Enter " + this.lang + " Source Code";
    let value = this.getAttribute("value") || this.innerHTML || "";
    
    this.innerHTML = ""; // Clear Content
    
    /* Create Textarea */
    let textarea = document.createElement("textarea");
    textarea.placeholder = placeholder;
    textarea.value = value;
    textarea.className = "code-input_editing";
    textarea.setAttribute("spellcheck", "false");
    
    if(this.getAttribute("name")) {
      textarea.setAttribute("name", this.getAttribute("name")); // for use in forms
      this.removeAttribute("name");
    }
    
    textarea.setAttribute("oninput", "codeInput.update(this.value, this.parentElement); codeInput.sync_scroll(this, this.parentElement);");
    textarea.setAttribute("onscroll", "codeInput.sync_scroll(this, this.parentElement);");
    textarea.setAttribute("onkeydown", "codeInput.check_tab(this, event);");
    
    this.append(textarea);
 
    /* Create pre code */
    let code = document.createElement("code");
    code.className = "code-input_highlighting-content language-" + lang; // Language for Prism.js
    code.innerText = value;
    
    let pre = document.createElement("pre");
    pre.className = "code-input_highlighting";
    pre.setAttribute("aria-hidden", "true"); // Hide for screen readers
    pre.append(code);
    this.append(pre);
    
    /* Add code from value attribute - useful for loading from backend */
    let text = this.value;
    let result_element = code
    
    // Handle final newlines (see article)
    // if(text[text.length-1] == "\n") {
    //   text += " ";
    // }
    
    // Update code
    result_element.innerHTML = text.replace(new RegExp("&", "g"), "&amp;").replace(new RegExp("<", "g"), "&lt;"); /* Global RegExp */
    
    // Syntax Highlight
    Prism.highlightElement(result_element);
  }
  static get observedAttributes() {
    return ["value", "placeholder"]; // Attributes to monitor
  }
  attributeChangedCallback(name, oldValue, newValue) {
    
    let textarea = this.getElementsByClassName("code-input_editing")[0];
    let result_element = this.getElementsByClassName("code-input_highlighting-content")[0];
    
    switch(name) {
        
      case "value":
        
        // Handle final newlines (see article)
        // if(text[text.length-1] == "\n") {
        //   text += " ";
        // }
        
        // Update code
        result_element.innerHTML = text.replace(new RegExp("&", "g"), "&amp;").replace(new RegExp("<", "g"), "&lt;"); /* Global RegExp */
        
        // Syntax Highlight
        Prism.highlightElement(result_element);
        
        break;
        
      case "placeholder":
        textarea.placeholder = oldValue;
        break;
    }
  }
}

customElements.define("code-input", CodeInput); // Set tag